@@ -22,6 +22,9 @@
 #ifndef __RB_FILE_HELPERS_H
 #define __RB_FILE_HELPERS_H
 
+#include <glib.h>
+#include <libgnomevfs/gnome-vfs.h>
+
 G_BEGIN_DECLS
 
 const char *	rb_file			(const char *filename);
@@ -30,6 +33,8 @@ const char *	rb_dot_dir		(void);
 
 void		rb_ensure_dir_exists	(const char *dir);
 
+GnomeVFSResult	rb_uri_mkstemp		(const char *prefix, char **uri, GnomeVFSHandle **handle);
+
 char *		rb_uri_resolve_symlink	(const char *uri);
 gboolean	rb_uri_is_directory	(const char *uri);
 gboolean	rb_uri_exists		(const char *uri);
