@@ -25,14 +25,17 @@
 #include <stdarg.h>
 #include <glib/gtypes.h>
 #include <glib-object.h>
+#include <gtk/gtkimage.h>
 
 G_BEGIN_DECLS
 
-gboolean rb_true_function (gpointer dummy, ...);
-gboolean rb_false_function (gpointer dummy, ...);
+gboolean rb_true_function (gpointer dummy);
+gboolean rb_false_function (gpointer dummy);
 
 gboolean rb_gvalue_compare (GValue *a, GValue *b);
 
+GtkWidget *rb_image_new_from_stock (const gchar *stock_id, GtkIconSize size);
+
 G_END_DECLS
 
 #endif /* __RB_UTIL_H */
